@@ -1,6 +1,8 @@
 ## ----echo=FALSE, include=FALSE------------------------------------------------
 knitr::opts_chunk$set(collapse = TRUE)
 if (file.exists("nc1.shp"))
+knitr::opts_chunk$set(fig.height = 4.5)
+knitr::opts_chunk$set(fig.width = 6)
 	file.remove("nc1.shp", "nc1.dbf", "nc1.shx")
 
 ## -----------------------------------------------------------------------------
