@@ -12,9 +12,40 @@
 #define NO_GDAL_CPP_HEADERS
 #include "gdal_sf_pkg.h"
 
-#if (!(GDAL_VERSION_MAJOR == 2 && GDAL_VERSION_MINOR < 1))
+#if GDAL_VERSION_NUM >= 2010000
 # include "gdal_utils.h" // requires >= 2.1
 
+/* modified from GDALTermProgress: */
+int CPL_STDCALL GDALRProgress( double dfComplete,
+                                  CPL_UNUSED const char * pszMessage,
+                                  CPL_UNUSED void * pProgressArg )
+{
+    const int nThisTick = std::min(40, std::max(0,
+        static_cast<int>(dfComplete * 40.0) ));
+
+    // Have we started a new progress run?
+    static int nLastTick = -1;
+    if( nThisTick < nLastTick && nLastTick >= 39 )
+        nLastTick = -1;
+
+    if( nThisTick <= nLastTick )
+        return TRUE;
+
+    while( nThisTick > nLastTick )
+    {
+        ++nLastTick;
+        if( nLastTick % 4 == 0 )
+            Rprintf("%d", (nLastTick / 4) * 10 );
+        else
+            Rprintf("." );
+    }
+
+    if( nThisTick == 40 )
+        Rprintf(" - done.\n" );
+
+    return TRUE;
+}
+
 // [[Rcpp::export]]
 Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options, 
 		Rcpp::CharacterVector oo) {
@@ -36,7 +67,8 @@ Rcpp::CharacterVector CPL_gdalinfo(Rcpp::CharacterVector obj, Rcpp::CharacterVec
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
+		bool quiet = true) {
 
 	int err = 0;
 
@@ -51,6 +83,12 @@ Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 	std::vector <char *> options_char = create_options(options, true);
 	GDALWarpAppOptions* opt = GDALWarpAppOptionsNew(options_char.data(), NULL);
 
+	if (! quiet) {
+		GDALWarpAppOptionsSetProgress(opt, GDALRProgress, NULL);
+#if GDAL_VERSION_NUM >= 2030000
+		GDALWarpAppOptionsSetQuiet(opt, 0);
+#endif
+	}
 	GDALDatasetH result = GDALWarp(dst_ds == NULL ? (const char *) dst[0] : NULL, dst_ds, 
 		src.size(), src_pt.data(), opt, &err);
 	GDALWarpAppOptionsFree(opt);
@@ -65,13 +103,15 @@ Rcpp::LogicalVector CPL_gdalwarp(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalrasterize(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
 		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
-		bool overwrite = false) {
+		bool overwrite = false, bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALRasterizeOptions* opt =  GDALRasterizeOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALRasterizeOptionsSetProgress(opt, GDALRProgress, NULL);
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, 
 		NULL, oo_char.data(), NULL);
 	if (src_pt == NULL)
@@ -97,36 +137,40 @@ Rcpp::LogicalVector CPL_gdalrasterize(Rcpp::CharacterVector src, Rcpp::Character
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdaltranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	std::vector <char *> oo_char = create_options(oo, true);
 	GDALTranslateOptions* opt =  GDALTranslateOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALTranslateOptionsSetProgress(opt, GDALRProgress, NULL);
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
 		NULL, oo_char.data(), NULL);
 	if (src_pt == NULL)
 		return 1; // #nocov
 	GDALDatasetH result = GDALTranslate((const char *) dst[0], src_pt, opt, &err);
 	GDALTranslateOptionsFree(opt);
-	/* see https://github.com/r-spatial/sf/issues/1352:
-	if (src_pt != NULL)
-		GDALClose(src_pt);
-	*/
+	// see https://github.com/r-spatial/sf/issues/1352:
 	if (result != NULL)
 		GDALClose(result);
+	if (src_pt != NULL)
+		GDALClose(src_pt);
 	return result == NULL || err;
 }
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalvectortranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
+		bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	GDALVectorTranslateOptions* opt =  GDALVectorTranslateOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALVectorTranslateOptionsSetProgress(opt, GDALRProgress, NULL);
 	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_VECTOR | GA_ReadOnly, NULL, 
 		oo_char.data(), NULL);
@@ -147,11 +191,13 @@ Rcpp::LogicalVector CPL_gdalvectortranslate(Rcpp::CharacterVector src, Rcpp::Cha
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalbuildvrt(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	GDALBuildVRTOptions* opt = GDALBuildVRTOptionsNew(options_char.data(), NULL);
+	if (! quiet)
+		GDALBuildVRTOptionsSetProgress(opt, GDALRProgress, NULL);
 	GDALDatasetH result = NULL;
 	if (oo.size()) { 
 		// to understand why we don't always take this path, read: 
@@ -181,14 +227,16 @@ Rcpp::LogicalVector CPL_gdalbuildvrt(Rcpp::CharacterVector src, Rcpp::CharacterV
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdaldemprocessing(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, Rcpp::CharacterVector colorfilename,
-		Rcpp::CharacterVector oo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector processing, 
+		Rcpp::CharacterVector colorfilename, Rcpp::CharacterVector oo, bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALDEMProcessingOptions* opt =  GDALDEMProcessingOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALDEMProcessingOptionsSetProgress(opt, GDALRProgress, NULL);
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
 		NULL, oo_char.data(), NULL);
 	if (src_pt == NULL)
@@ -207,7 +255,8 @@ Rcpp::LogicalVector CPL_gdaldemprocessing(Rcpp::CharacterVector src, Rcpp::Chara
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo, 
+		bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
@@ -215,6 +264,8 @@ Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::Character
 	std::vector <char *> doo_char = create_options(doo, true); // open options
 	GDALNearblackOptions* opt =  GDALNearblackOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALNearblackOptionsSetProgress(opt, GDALRProgress, NULL);
 	// GDALDatasetH src_pt = GDALOpen((const char *) src[0], GA_ReadOnly);
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, NULL, oo_char.data(), NULL);
 	GDALDatasetH dst_pt = GDALOpenEx((const char *) dst[0], GDAL_OF_RASTER | GA_Update, NULL, doo_char.data(), NULL);
@@ -229,13 +280,16 @@ Rcpp::LogicalVector CPL_gdalnearblack(Rcpp::CharacterVector src, Rcpp::Character
 
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
-		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo,
+		bool quiet = true) {
 
 	int err = 0;
 	std::vector <char *> options_char = create_options(options, true);
 	std::vector <char *> oo_char = create_options(oo, true); // open options
 	GDALGridOptions* opt =  GDALGridOptionsNew(options_char.data(), NULL);
 
+	if (! quiet)
+		GDALGridOptionsSetProgress(opt, GDALRProgress, NULL);
 	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_ALL | GA_ReadOnly, 
 		NULL, oo_char.data(), NULL);
 	GDALDatasetH result = GDALGrid((const char *) dst[0], src_pt, opt, &err);
@@ -247,6 +301,70 @@ Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 	return result == NULL || err;
 }
 
+// gdal >= 3.1: mdim utils:
+#if GDAL_VERSION_NUM >= 3010000
+// [[Rcpp::export]]
+Rcpp::CharacterVector CPL_gdalmdiminfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options,
+		Rcpp::CharacterVector oo) { 
+	std::vector <char *> oo_char = create_options(oo, true);
+	GDALDatasetH ds = GDALOpenEx((const char *) obj[0], GA_ReadOnly, NULL, oo_char.data(), NULL);
+	if (ds == NULL)
+		return 1; // #nocov
+	std::vector <char *> options_char = create_options(options, true);
+	GDALMultiDimInfoOptions* opt = GDALMultiDimInfoOptionsNew(options_char.data(), NULL);
+	char *ret_val = GDALMultiDimInfo(ds, opt);
+	GDALMultiDimInfoOptionsFree(opt);
+	GDALClose(ds);
+	Rcpp::CharacterVector ret(1);
+	if (ret_val != NULL) {
+		ret[0] = ret_val; // copies?
+		CPLFree(ret_val);
+	} else
+		Rcpp::stop("GDALMultiDimInfo returned NULL");
+	return ret;
+}
+
+// [[Rcpp::export]]
+Rcpp::LogicalVector CPL_gdalmdimtranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, bool quiet = true) {
+
+	int err = 0;
+	std::vector <char *> options_char = create_options(options, true);
+	std::vector <char *> oo_char = create_options(oo, true);
+	GDALMultiDimTranslateOptions* opt = GDALMultiDimTranslateOptionsNew(options_char.data(), NULL);
+
+	if (! quiet)
+		GDALMultiDimTranslateOptionsSetProgress(opt, GDALRProgress, NULL);
+	GDALDatasetH src_pt = GDALOpenEx((const char *) src[0], GDAL_OF_RASTER | GA_ReadOnly, 
+		NULL, oo_char.data(), NULL);
+	if (src_pt == NULL)
+		return 1; // #nocov
+	std::vector<GDALDatasetH> srcpt(src.size());
+	for (int i = 0; i < src.size(); i++) {
+		srcpt[i] = GDALOpenEx((const char *) src[i], GDAL_OF_RASTER | GA_ReadOnly, NULL, 
+			oo_char.data(), NULL);
+		if (srcpt[i] == NULL)
+			Rcpp::stop("cannot open source dataset");
+	}
+	GDALDatasetH result = GDALMultiDimTranslate((const char *) dst[0], NULL, srcpt.size(), srcpt.data(), opt, &err);
+	GDALMultiDimTranslateOptionsFree(opt);
+	if (result != NULL)
+		GDALClose(result);
+	for (int i = 0; i < src.size(); i++)
+		GDALClose(srcpt[i]);
+	return result == NULL || err;
+}
+#else
+Rcpp::CharacterVector CPL_gdalmdiminfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options, 
+		Rcpp::CharacterVector oo) {
+	Rcpp::stop("GDAL version >= 3.1 required for mdiminfo");
+}
+Rcpp::LogicalVector CPL_gdalmdimtranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo, bool quiet = true) {
+	Rcpp::stop("GDAL version >= 3.1 required for mdimtranslate");
+}
+#endif
+
 #else
 #include "Rcpp.h"
 
@@ -296,82 +414,103 @@ Rcpp::LogicalVector CPL_gdalgrid(Rcpp::CharacterVector src, Rcpp::CharacterVecto
 		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
 	Rcpp::stop("GDAL version >= 2.1 required for gdal_utils");
 }
+
+Rcpp::CharacterVector CPL_gdalmdiminfo(Rcpp::CharacterVector obj, Rcpp::CharacterVector options, 
+		Rcpp::CharacterVector oo) {
+	Rcpp::stop("GDAL version >= 3.1 required for mdiminfo");
+}
+
+Rcpp::LogicalVector CPL_gdalmdimtranslate(Rcpp::CharacterVector src, Rcpp::CharacterVector dst,
+		Rcpp::CharacterVector options, Rcpp::CharacterVector oo) {
+	Rcpp::stop("GDAL version >= 3.1 required for mdimtranslate");
+}
 #endif
 
 // #nocov start
-// https://www.gdal.org/warptut.html :
+// https://gdal.org/tutorials/warp_tut.html
 // [[Rcpp::export]]
 Rcpp::LogicalVector CPL_gdal_warper(Rcpp::CharacterVector infile, Rcpp::CharacterVector outfile,
-		Rcpp::IntegerVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo) {
+		Rcpp::IntegerVector options, Rcpp::CharacterVector oo, Rcpp::CharacterVector doo,
+		bool quiet = true) {
 
 	std::vector <char *> oo_char = create_options(oo, true); // open options
-    GDALDatasetH  hSrcDS, hDstDS;
-    // Open input and output files.
-    GDALAllRegister();
-    hSrcDS = GDALOpenEx( infile[0], GA_ReadOnly, NULL, oo_char.data(), NULL );
+	GDALDatasetH  hSrcDS, hDstDS;
+	// Open input and output files.
+	GDALAllRegister();
+	hSrcDS = GDALOpenEx( infile[0], GA_ReadOnly, NULL, oo_char.data(), NULL );
 	if (hSrcDS == NULL)
 		Rcpp::stop("input file not found");
 	std::vector <char *> doo_char = create_options(doo, true); // open options
-    hDstDS = GDALOpenEx(outfile[0], GA_Update, NULL, doo_char.data(), NULL);
+	hDstDS = GDALOpenEx(outfile[0], GA_Update, NULL, doo_char.data(), NULL);
 	if (hDstDS == NULL)
 		Rcpp::stop("could not open output file for writing");
-    // Setup warp options.
-    GDALWarpOptions *psWarpOptions = GDALCreateWarpOptions();
-    psWarpOptions->hSrcDS = hSrcDS;
-    psWarpOptions->hDstDS = hDstDS;
+	// Setup warp options.
+	GDALWarpOptions *psWarpOptions = GDALCreateWarpOptions();
+	psWarpOptions->hSrcDS = hSrcDS;
+	psWarpOptions->hDstDS = hDstDS;
 
-    psWarpOptions->nBandCount = 0;
+	if (GDALGetRasterCount(hSrcDS) != GDALGetRasterCount(hDstDS))
+		Rcpp::stop("warper: source and destination must have the same number of bands");
 
-	if (GDALGetRasterCount(hSrcDS) > GDALGetRasterCount(hDstDS))
-		Rcpp::stop("warper: source has more bands than destination");
+	psWarpOptions->nBandCount = GDALGetRasterCount(hSrcDS);
+	psWarpOptions->panSrcBands =
+		(int *) CPLMalloc(sizeof(int) * psWarpOptions->nBandCount );
+	psWarpOptions->panDstBands =
+		(int *) CPLMalloc(sizeof(int) * psWarpOptions->nBandCount );
+	for (size_t i = 0; i < psWarpOptions->nBandCount; i++) {
+		psWarpOptions->panSrcBands[i] = i + 1;
+		psWarpOptions->panDstBands[i] = i + 1;
+	}
 
-    psWarpOptions->padfSrcNoDataReal = (double *) CPLMalloc(sizeof(double) * GDALGetRasterCount(hSrcDS));
-    psWarpOptions->padfDstNoDataReal = (double *) CPLMalloc(sizeof(double) * GDALGetRasterCount(hSrcDS));
+	psWarpOptions->padfSrcNoDataReal = (double *) CPLMalloc(sizeof(double) * GDALGetRasterCount(hSrcDS));
+	psWarpOptions->padfDstNoDataReal = (double *) CPLMalloc(sizeof(double) * GDALGetRasterCount(hSrcDS));
 
-    GDALRasterBandH poBand;
+	GDALRasterBandH poBand;
 	int success;
 	double d = 0xffffffff;
 	for (int i = 0; i < GDALGetRasterCount(hSrcDS); i++) {
-    	poBand = GDALGetRasterBand(hSrcDS, i + 1);
-    	GDALGetRasterNoDataValue(poBand, &success);
+		poBand = GDALGetRasterBand(hSrcDS, i + 1);
+		GDALGetRasterNoDataValue(poBand, &success);
 		if (success)
-    		psWarpOptions->padfSrcNoDataReal[i] = GDALGetRasterNoDataValue(poBand, &success);
-    		// Rcpp::Rcout << GDALGetRasterNoDataValue(poBand, &success) << std::endl;
+			psWarpOptions->padfSrcNoDataReal[i] = GDALGetRasterNoDataValue(poBand, &success);
+			// Rcpp::Rcout << GDALGetRasterNoDataValue(poBand, &success) << std::endl;
 		else
 			memcpy(&(psWarpOptions->padfSrcNoDataReal[i]), &d, sizeof(double));
-    	poBand = GDALGetRasterBand(hDstDS, i + 1);
-    	GDALGetRasterNoDataValue(poBand, &success);
+		poBand = GDALGetRasterBand(hDstDS, i + 1);
+		GDALGetRasterNoDataValue(poBand, &success);
 		if (success)
-    		psWarpOptions->padfDstNoDataReal[0] = GDALGetRasterNoDataValue(poBand, &success);
-    		// Rcpp::Rcout << GDALGetRasterNoDataValue(poBand, &success) << std::endl;
+			psWarpOptions->padfDstNoDataReal[0] = GDALGetRasterNoDataValue(poBand, &success);
+			// Rcpp::Rcout << GDALGetRasterNoDataValue(poBand, &success) << std::endl;
 		else // NaN:
 			memcpy(&(psWarpOptions->padfDstNoDataReal[i]), &d, sizeof(double));
 	}
 
-    // psWarpOptions->pfnProgress = GDALTermProgress; // 0...10...20...30...40...50...60...70...80...90...100 - done.
-    psWarpOptions->pfnProgress = GDALDummyProgress;
-    // Establish reprojection transformer.
+	if (quiet)
+		psWarpOptions->pfnProgress = GDALDummyProgress;
+	else
+		psWarpOptions->pfnProgress = GDALRProgress; // 0...10...20...30...40...50...60...70...80...90...100 - done.
+	// Establish reprojection transformer.
 	if (options.size() == 1)
 		psWarpOptions->eResampleAlg = (GDALResampleAlg) options[0];
-    psWarpOptions->pTransformerArg =
-        GDALCreateGenImgProjTransformer( hSrcDS,
-                                         GDALGetProjectionRef(hSrcDS),
-                                         hDstDS,
-                                         GDALGetProjectionRef(hDstDS),
-                                         FALSE, 0.0, 1 );
-    psWarpOptions->pfnTransformer = GDALGenImgProjTransform;
-    // Initialize and execute the warp operation.
-    GDALWarpOperation oOperation;
-    oOperation.Initialize( psWarpOptions );
-    oOperation.ChunkAndWarpImage( 0, 0,
-                                  GDALGetRasterXSize( hDstDS ),
-                                  GDALGetRasterYSize( hDstDS ) );
-    GDALDestroyGenImgProjTransformer( psWarpOptions->pTransformerArg );
-    GDALDestroyWarpOptions( psWarpOptions );
-    if (hDstDS)
+	psWarpOptions->pTransformerArg =
+		GDALCreateGenImgProjTransformer( hSrcDS,
+										 GDALGetProjectionRef(hSrcDS),
+										 hDstDS,
+										 GDALGetProjectionRef(hDstDS),
+										 FALSE, 0.0, 1 );
+	psWarpOptions->pfnTransformer = GDALGenImgProjTransform;
+	// Initialize and execute the warp operation.
+	GDALWarpOperation oOperation;
+	oOperation.Initialize( psWarpOptions );
+	oOperation.ChunkAndWarpImage( 0, 0,
+								  GDALGetRasterXSize( hDstDS ),
+								  GDALGetRasterYSize( hDstDS ) );
+	GDALDestroyGenImgProjTransformer( psWarpOptions->pTransformerArg );
+	GDALDestroyWarpOptions( psWarpOptions );
+	if (hDstDS)
 		GDALClose( hDstDS );
-    if (hSrcDS)
+	if (hSrcDS)
 		GDALClose( hSrcDS );
-    return false;
+	return false;
 }
 // #nocov end
