@@ -31,6 +31,9 @@ test_that("CPL_geos_is_valid works", {
 })
 
 test_that("geos ops give warnings and errors on longlat", {
+
+    skip_if_not(!sf_use_s2())
+
 	nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
 	x = nc[1:2,]
 	y = nc[2:3,]
@@ -39,15 +42,15 @@ test_that("geos ops give warnings and errors on longlat", {
 	l = st_sfc(st_linestring(matrix(1:10, ncol=2)), crs = st_crs(nc))
 	expect_silent(st_polygonize(l))
 
-	#expect_message(st_intersects(x,y))
-	#expect_message(st_disjoint(x,y))
-	#expect_message(st_touches(x,y))
+	expect_message(st_intersects(x,y))
+	expect_message(st_disjoint(x,y))
+	expect_message(st_touches(x,y))
 	expect_message(st_crosses(x,y))
-	#expect_message(st_within(x,y))
-	#expect_message(st_contains(x,y))
+	expect_message(st_within(x,y))
+	expect_message(st_contains(x,y))
 	expect_message(st_overlaps (x,y))
-	#expect_message(st_covers(x,y))
-	#expect_message(st_covered_by(x,y))
+	expect_message(st_covers(x,y))
+	expect_message(st_covered_by(x,y))
 
 	expect_warning(st_buffer(x, .1))
 	expect_warning(st_buffer(x, .1, joinStyle = "BEVEL"))
